// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package org.apache.impala.analysis;

import org.apache.impala.authorization.Privilege;
import org.apache.impala.catalog.Catalog;
import org.apache.impala.common.AnalysisException;
import org.apache.impala.thrift.TUseDbParams;

/**
 * Representation of a USE db statement.
 */
public class UseStmt extends StatementBase {
  private final String database_;

  public UseStmt(String db) {
    database_ = db;
  }

  public String getDatabase() { return database_; }

  @Override
  public String toSql() {
    return "USE " + database_;
  }

  @Override
  public void analyze(Analyzer analyzer) throws AnalysisException {
    if (!database_.equalsIgnoreCase(Catalog.DEFAULT_DB)) {
      // USE <default> should always be allowed.
      analyzer.getDb(database_, Privilege.ANY, true);
    }
  }

  public TUseDbParams toThrift() {
    TUseDbParams params = new TUseDbParams();
    params.setDb(getDatabase());
    return params;
  }
}
