// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package org.apache.impala.planner;

import org.apache.impala.common.Id;
import org.apache.impala.common.IdGenerator;

public class JoinTableId extends Id<JoinTableId> {
  static int INVALID_JOIN_TABLE_ID = -1;

  // Construction only allowed via an IdGenerator.
  protected JoinTableId(int id) {
    super(id);
  }

  public static JoinTableId INVALID;
  static {
    INVALID = new JoinTableId(INVALID_JOIN_TABLE_ID);
  }

  public boolean isValid() { return id_ != INVALID_JOIN_TABLE_ID; }

  public static IdGenerator<JoinTableId> createGenerator() {
    return new IdGenerator<JoinTableId>() {
      @Override
      public JoinTableId getNextId() { return new JoinTableId(nextId_++); }
      @Override
      public JoinTableId getMaxId() { return new JoinTableId(nextId_ - 1); }
    };
  }

  @Override
  public String toString() {
    return String.format("%02d", id_);
  }
}
